//-------------------------------------------------------------------------------------------------
// <copyright file="resrutil.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
// 
// <summary>
//    Resource reading helper functions.
// </summary>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"


/********************************************************************
ResReadString

NOTE: ppwzString should be freed with StrFree()
********************************************************************/
extern "C" HRESULT DAPI ResReadString(
    __in HINSTANCE hinst,
    __in UINT uID,
    __out LPWSTR* ppwzString
    )
{
    Assert(hinst && ppwzString);

    HRESULT hr = S_OK;
    LPWSTR pwz = NULL;
    DWORD cch = 64;  // first guess
    DWORD cchReturned = 0;

    do
    {
        hr = StrAlloc(&pwz, cch);
        ExitOnFailureDebugTrace1(hr, "failed to allocate string for resource id: %d", uID);

        cchReturned = ::LoadStringW(hinst, uID, pwz, cch);
        if (0 == cchReturned)
            ExitOnLastErrorDebugTrace1(hr, "failed to load string resource id: %d", uID);

        // if the returned string count is one character too small, it's likely we have
        // more data to read
        if (cchReturned + 1 == cch)
        {
            cch *= 2;
            hr = S_FALSE;
        }
    } while (S_FALSE == hr);
    ExitOnFailureDebugTrace1(hr, "failed to load string resource id: %d", uID);

    *ppwzString = pwz;
    pwz = NULL;
    Assert(S_OK == hr);

LExit:
    ReleaseStr(pwz);
    return hr;
}


/********************************************************************
ResReadData - returns a pointer to the specified resource data

NOTE:  there is no "free" function for this call
********************************************************************/
extern "C" HRESULT DAPI ResReadData(
    __in LPCSTR szDataName,
    __out PVOID *ppv,
    __out DWORD *pcb
    )
{
    Assert(szDataName);
    Assert(ppv);

    HRESULT hr = S_OK;
    HRSRC hRsrc = NULL;
    HGLOBAL hData = NULL;
    DWORD cbData = 0;

    hRsrc = ::FindResourceExA(NULL, RT_RCDATA, szDataName, MAKELANGID(LANG_NEUTRAL, SUBLANG_NEUTRAL));
    ExitOnNullWithLastError(hRsrc, hr, "Failed to find resource.");

    hData = ::LoadResource(NULL, hRsrc);
    ExitOnNullWithLastError(hData, hr, "Failed to load resource.");

    cbData = ::SizeofResource(NULL, hRsrc);
    if (!cbData)
    {
        ExitWithLastError(hr, "Failed to get size of resource.");
    }

    *ppv = ::LockResource(hData);
    ExitOnNullWithLastError(*ppv, hr, "Failed to lock data resource.");
    *pcb = cbData;

LExit:
    return hr;
}


/********************************************************************
ResExportDataToFile - extracts the resource data to the specified target file

********************************************************************/
extern "C" HRESULT DAPI ResExportDataToFile(
    __in LPCSTR szDataName,
    __in LPCWSTR wzTargetFile,
    __in DWORD dwCreationDisposition
    )
{
    HRESULT hr = S_OK;
    PVOID pData = NULL;
    DWORD cbData = 0;
    DWORD cbWritten = 0;
    HANDLE hFile = INVALID_HANDLE_VALUE;
    BOOL bCreatedFile = FALSE;

    hr = ResReadData(szDataName, &pData, &cbData);
    ExitOnFailure1(hr, "Failed to GetData from %s.", szDataName);

    hFile = ::CreateFileW(wzTargetFile, GENERIC_WRITE, 0, NULL, dwCreationDisposition, FILE_ATTRIBUTE_NORMAL, NULL);
    if (INVALID_HANDLE_VALUE == hFile)
    {
        ExitWithLastError1(hr, "Failed to CreateFileW for %S.", wzTargetFile);
    }
    bCreatedFile = TRUE;

    if (!::WriteFile(hFile, pData, cbData, &cbWritten, NULL))
    {
        ExitWithLastError1(hr, "Failed to ::WriteFile for %S.", wzTargetFile);
    }

LExit:
    if (INVALID_HANDLE_VALUE != hFile)
    {
        ::CloseHandle(hFile);
    }

    if (FAILED(hr))
    {
        if (bCreatedFile)
        {
            ::DeleteFileW(wzTargetFile);
        }
    }

    return hr;
}
