//-------------------------------------------------------------------------------------------------
// <copyright file="UtilHarvesterMutator.cs" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
// 
// <summary>
// The Windows Installer XML Toolset harvester mutator.
// </summary>
//-------------------------------------------------------------------------------------------------

namespace Microsoft.Tools.WindowsInstallerXml.Extensions
{
    using System;
    using System.Collections;
    using System.IO;
    using System.Runtime.InteropServices;

    using Wix = Microsoft.Tools.WindowsInstallerXml.Serialize;

    /// <summary>
    /// The Windows Installer XML Toolset harvester mutator.
    /// </summary>
    public sealed class UtilHarvesterMutator : MutatorExtension
    {
        /// <summary>
        /// enable the RegisterTypeLIb API to use the appropriate override mapping for non-admin users on Vista
        /// </summary>
        [DllImport("Oleaut32.dll")]
        private static extern void OaEnablePerUserTLibRegistration();

        /// <summary>
        /// Gets the sequence of this mutator extension.
        /// </summary>
        /// <value>The sequence of this mutator extension.</value>
        public override int Sequence
        {
            get { return 100; }
        }

        /// <summary>
        /// Mutate a WiX document.
        /// </summary>
        /// <param name="wix">The Wix document element.</param>
        public override void Mutate(Wix.Wix wix)
        {
            this.MutateElement(null, wix);
        }

        /// <summary>
        /// Mutate an element.
        /// </summary>
        /// <param name="parentElement">The parent of the element to mutate.</param>
        /// <param name="element">The element to mutate.</param>
        private void MutateElement(Wix.IParentElement parentElement, Wix.ISchemaElement element)
        {
            if (element is Wix.File)
            {
                this.MutateFile(parentElement, (Wix.File)element);
            }

            // mutate the child elements
            if (element is Wix.IParentElement)
            {
                ArrayList childElements = new ArrayList();

                // copy the child elements to a temporary array (to allow them to be deleted/moved)
                foreach (Wix.ISchemaElement childElement in ((Wix.IParentElement)element).Children)
                {
                    childElements.Add(childElement);
                }

                foreach (Wix.ISchemaElement childElement in childElements)
                {
                    this.MutateElement((Wix.IParentElement)element, childElement);
                }
            }
        }

        /// <summary>
        /// Mutate a file.
        /// </summary>
        /// <param name="parentElement">The parent of the element to mutate.</param>
        /// <param name="file">The file to mutate.</param>
        private void MutateFile(Wix.IParentElement parentElement, Wix.File file)
        {
            if (null != file.Source)
            {
                string fileExtension = Path.GetExtension(file.Source);

                if (0 == String.Compare(".ax", fileExtension, true) || // DirectShow filter
                    0 == String.Compare(".dll", fileExtension, true) ||
                    0 == String.Compare(".exe", fileExtension, true) ||
                    0 == String.Compare(".ocx", fileExtension, true) || // ActiveX
                    0 == String.Compare(".olb", fileExtension, true) || // type library
                    0 == String.Compare(".tlb", fileExtension, true)) // type library
                {
                    // try the assembly harvester
                    try
                    {
                        AssemblyHarvester assemblyHarvester = new AssemblyHarvester();

                        Wix.RegistryValue[] registryValues = assemblyHarvester.HarvestRegistryValues(file.Source);

                        foreach (Wix.RegistryValue registryValue in registryValues)
                        {
                            parentElement.AddChild(registryValue);
                        }
                    }
                    catch
                    {
                        // try the self-reg harvester
                        try
                        {
                            DllHarvester dllHarvester = new DllHarvester();

                            Wix.RegistryValue[] registryValues = dllHarvester.HarvestRegistryValues(file.Source);

                            foreach (Wix.RegistryValue registryValue in registryValues)
                            {
                                parentElement.AddChild(registryValue);
                            }
                        }
                        catch
                        {
                            // Remember whether we were able to call OaEnablePerUserTLibRegistration
                            bool calledPerUserTLibReg = false;

                            // try the type library harvester
                            try
                            {
                                try
                                {
                                    OaEnablePerUserTLibRegistration();
                                    calledPerUserTLibReg = true;
                                }
                                catch (EntryPointNotFoundException)
                                {
                                }

                                TypeLibraryHarvester typeLibHarvester = new TypeLibraryHarvester();

                                Wix.RegistryValue[] registryValues = typeLibHarvester.HarvestRegistryValues(file.Source);

                                foreach (Wix.RegistryValue registryValue in registryValues)
                                {
                                    parentElement.AddChild(registryValue);
                                }
                            }
                            catch (COMException ce)
                            {
                                //  0x8002801C (TYPE_E_REGISTRYACCESS)
                                // If we don't have permission to harvest typelibs, it's likely because we're on
                                // Vista or higher and aren't an Admin, or don't have the appropriate QFE installed.
                                if (!calledPerUserTLibReg && (0x8002801c == unchecked((uint)ce.ErrorCode)))
                                {
                                    this.Core.OnMessage(WixWarnings.InsufficientPermissionHarvestTypeLib());
                                }
                            }
                            catch
                            {
                                // ignore all exceptions
                            }
                        }
                    }
                }
            }
        }
    }
}