//-------------------------------------------------------------------------------------------------
// <copyright file="WixReferenceValidator.cs" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
// 
// <summary>
// Contains the WixReferenceValidator class.
// </summary>
//-------------------------------------------------------------------------------------------------

namespace Microsoft.Tools.WindowsInstallerXml.VisualStudio
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Diagnostics.CodeAnalysis;
    using System.IO;
    using System.Globalization;
    using System.Reflection;
    using System.Security;
    using System.Xml;
    using System.Xml.Schema;
    using Microsoft.Tools.WindowsInstallerXml;

    /// <summary>
    /// Contains useful helper methods.
    /// </summary>
    internal static class WixReferenceValidator
    {
        /// <summary>
        /// Checks to see if an assemlby to see if an assembly is a valid WixExtension.
        /// </summary>
        /// <param name="assemblyPath">Path to the assembly.</param>
        /// <param name="extensionDependencyFolders">Dependent assembly path</param>/> 
        /// <returns>True if it is a valid Wix extension</returns>
        /// <remarks>
        /// <paramref name="assemblyPath"/> can be in several different forms:
        /// <list type="number">
        /// <item><term>Absolute path to an assembly (C:\MyExtensions\ExtensionAssembly.dll)</term></item>
        /// <item><term>Relative path to an assembly (..\..\MyExtensions\ExtensionAssembly.dll)</term></item>
        /// </list>
        /// </remarks>
        internal static bool IsValidWixExtension(string assemblyPath, params string[] extensionDependencyFolders)
        {
            if (String.IsNullOrEmpty(assemblyPath))
            {
                return false;
            }

            WixExtensionValidator extensionValidator = new WixExtensionValidator(extensionDependencyFolders);
            return extensionValidator.IsValid(assemblyPath);
        }

        /// <summary>
        /// Validates the Wix library.
        /// </summary>
         /// <param name="filePath">Path to the Wix library.</param>
        /// <returns>Returns true if it is a valid Wix Library.</returns>
        [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        internal static bool IsValidWixLibrary(string filePath)
        {
            if (String.IsNullOrEmpty(filePath))
            {
                return false;
            }

            // read the xml
            try
            {
                using (XmlReader reader = new XmlTextReader(filePath))
                {
                  reader.MoveToContent();
  
                  if (reader.LocalName != "wixLibrary")
                  {
                      return false;
                  }
                }
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Helper class to load an assembly and its dependents to inspect the types in the assembly.
        /// </summary>
        private class WixExtensionValidator
        {
            private string[] extensionDependencyFolders;

            /// <summary>
            /// Constructor
            /// </summary>
            /// <param name="dependencyProbeFolders">Folders to probe for dependencies</param>
            public WixExtensionValidator(string[] dependencyProbeFolders)
            {
                this.extensionDependencyFolders = dependencyProbeFolders;
            }

            /// <summary>
            /// Checks to see if there is a single type that is a sub class of
            /// Microsoft.Tools.WindowsInstallerXml.WixExtension
            /// </summary>
            /// <param name="assemblyPath">Path of the assembly.</param>
            /// <returns>True if it is a valid Wix extension</returns>
            /// <remarks>
            /// <paramref name="assemblyPath"/> can be in several different forms:
            /// <list type="number">
            /// <item><term>Absolute path to an assembly (C:\MyExtensions\ExtensionAssembly.dll)</term></item>
            /// <item><term>Relative path to an assembly (..\..\MyExtensions\ExtensionAssembly.dll)</term></item>
            /// </list>
            /// </remarks>
            [SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
            [SuppressMessage("Microsoft.Reliability", "CA2001:AvoidCallingProblematicMethods")]
            public bool IsValid(string assemblyPath)
            {
                if (String.IsNullOrEmpty(assemblyPath))
                {
                    return false;
                }

                bool result = false;

                AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(this.OnCurrentDomainAssemblyResolve);

                try
                {
                    Assembly extensionAssembly = Assembly.LoadFrom(assemblyPath);

                    AssemblyDefaultWixExtensionAttribute extensionAttribute = (AssemblyDefaultWixExtensionAttribute)Attribute.GetCustomAttribute(extensionAssembly, typeof(AssemblyDefaultWixExtensionAttribute));
                    Type extensionType = null;

                    if (extensionAttribute != null)
                    {
                        extensionType = extensionAttribute.ExtensionType;
                    }

                    if (extensionType != null)
                    {
                        if (extensionType.IsSubclassOf(typeof(WixExtension)))
                        {
                            result = true;
                        }
                    }
                }
                catch
                {
                }
                finally
                {
                    AppDomain.CurrentDomain.AssemblyResolve -= new ResolveEventHandler(this.OnCurrentDomainAssemblyResolve);
                }

                return result;
            }

            /// <summary>
            /// This event handler is invoked when a required assembly is not in the current folder.
            /// </summary>
            /// <param name="sender">Event originator</param>
            /// <param name="args">Resolve event arguments</param>
            /// <returns>Returns loaded assembly or null when it can't resolve/find the dependent assembly</returns>
            [SuppressMessage("Microsoft.Reliability", "CA2001:AvoidCallingProblematicMethods")]
            private Assembly OnCurrentDomainAssemblyResolve(object sender, ResolveEventArgs args)
            {
                Assembly currentAssembly = Assembly.GetExecutingAssembly();
                if (args.Name == currentAssembly.GetName().FullName)
                {
                    return currentAssembly;
                }

                if (this.extensionDependencyFolders == null || this.extensionDependencyFolders.Length == 0)
                {
                    return null;
                }

                AssemblyName referenceAssemblyName = new AssemblyName(args.Name);

                foreach (string folder in this.extensionDependencyFolders)
                {
                    string assemblyPathWithoutExtension = Path.Combine(folder, referenceAssemblyName.Name);
                    string assemblyPath = assemblyPathWithoutExtension + ".dll";

                    if (!File.Exists(assemblyPath))
                    {
                        assemblyPath = assemblyPathWithoutExtension + ".exe";

                        if (!File.Exists(assemblyPath))
                        {
                            continue;
                        }
                    }

                    try
                    {
                        AssemblyName definitionAssemblyName = AssemblyName.GetAssemblyName(assemblyPath);
                        if (AssemblyName.ReferenceMatchesDefinition(referenceAssemblyName, definitionAssemblyName))
                        {
                            return Assembly.LoadFrom(assemblyPath);
                        }
                    }
                    catch (ArgumentException)
                    {
                    }
                    catch (SecurityException)
                    {
                    }
                    catch (FileLoadException)
                    {
                    }
                }

                return null;
            }
        }
    }
}